<?php

/**
 * mo_viaebay__client
 * 
 * handle communication with viaebay
 * 
 * @version   $Id: mo_viaebay__client.php 204 2013-01-07 15:53:37Z $
 */

/**
 * handle communication with viaebay
 * 
 * @package   mo_viaebay 
 */
class mo_viaebay__client
{

  protected $DEBUG = false;
    
  /**
   * construct
   * 
   * @param oxSession $oxSession 
   * @param mo_viaebay__logger $logger 
   */
  public function __construct($oxSession, mo_viaebay__logger $logger)
  {
    $this->oxSession = $oxSession;
    $this->logger = $logger;
  }

  /**
   * call viaebay API with given parameters
   * 
   * @param array $params
   * @return string 
   */
  public function callService($params)
  {
    $params = $this->encodeParams($params);
    
    //get calling function
    $caller = false;
    $backtrace = debug_backtrace();
    if (!empty($backtrace[3])) {
       $caller = $backtrace[3]['class'] . '::' . $backtrace[3]['function'];
    }
    $this->logger->debug('Request(' . $caller. '): ' . var_export($params, true));

    if ($this->DEBUG) {
        ob_start();
        $out = fopen('php://output', 'w');
    }
    
    $ch = curl_init();
    $this->setHeaders($ch, $params);
    curl_setopt($ch, CURLOPT_URL, $params['url']);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    
    if ($this->DEBUG) {
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        curl_setopt($ch, CURLOPT_STDERR, $out);
    }
    curl_setopt($ch, CURLOPT_HEADER, 1);
    curl_setopt($ch, CURLOPT_TIMEOUT, 60);

    switch (strtoupper($params['verb']))
    {
      case 'GET':
        $result = $this->executeGet($ch);
        break;
      case 'POST':
        $result = $this->executePost($ch, $params);
        break;
      case 'PUT':
        $result = $this->executePut($ch, $params);
        break;
      case 'DELETE':
        $result = $this->executeDelete($ch, $params);
        break;
    }

    curl_close($ch);
    
    if ($this->DEBUG) {
        fclose($out);
        $this->logger->debug(ob_get_clean());
    }
    
    $this->logger->debug('Response(' . $caller . '): ' . var_export($result, true));
    return $result;
  }

  /**
   * check if utf8 encoding is chosen
   * 
   * @param array $params
   * @return array 
   */
  protected function encodeParams($params)
  {
    //convert from ISO to utf8
    if (oxRegistry::get('oxconfig')->getConfigParam('mo_viaebay__encode_utf8'))
    {
      $params = mo_viaebay__main::getInstance()->getHelper()->convertCharset('ISO-8859-1', 'UTF-8', $params);
    }
    
    return $params;
  }

  /**
   * set headers
   * 
   * @param curlHandler $ch
   * @param array $params 
   */
  protected function setHeaders(&$ch, $params)
  {
    $headers = array();
    $headers[] = 'Content-Type: ' . $params['request_format']['content_type'];
    $headers[] = 'Accept: application/json'; 
    $headers[] = 'Expect:'; //prvent 100-status code: see http://pilif.github.io/2007/02/the-return-of-except-100-continue/

    //send content-length on empty post requests (otherwise on .pt :HTTP/1.1 411 Length Required)
    if (empty($params['request_params']) && $params['verb'] == 'POST') {
       $headers[]  = 'Content-Length: 0';
    }
    
    if (!empty($params['auth_params']))
    {
      foreach ($params['auth_params'] as $key => $value)
      {
        $headers[] = $key . ': ' . $value;
      }
    }

    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
  }

  /**
   * execute curl operation
   *
   * @param curlHandler $ch
   * @return array 
   */
  protected function curlExec(&$ch)
  {
    $response = curl_exec($ch);

    if ($response === false)
    {
      $this->logger->error('Curl Error: ' . curl_error($ch));
    }

    $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
    $header      = substr($response, 0, $header_size);
    $body        = substr($response, $header_size);

    return array('header' => $header, 'body'   => $body);
  }

  /**
   * build request-body
   *
   * @param array $params
   * @return array 
   */
  public function buildRequestBody($params)
  {
    if (empty($params['request_format']['ident']))
    {
      return array();
    }

    if (empty($params['request_params']))
    {
      return array();
    }

    switch ($params['request_format']['ident'])
    {
      case 'json':
        return json_encode($params['request_params']);
    }
  }

  /**
   * execute with request-metod: GET
   *
   * @param curlHandler $ch
   * @return mixed 
   */
  protected function executeGet(&$ch)
  {
    return $this->curlExec($ch);
  }

  /**
   * execute with request-metod: POST
   *
   * @param curlHandler $ch
   * @param array $params
   * @return mixed 
   */
  protected function executePost($ch, $params)
  {
    $requestBody = $this->buildRequestBody($params);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $requestBody);
    curl_setopt($ch, CURLOPT_POST, 1);

    return $this->curlExec($ch);
  }

  /**
   * execute with request-metod: PUT
   *
   * @param curlHandler $ch
   * @param array $params
   * @return mixed 
   */
  protected function executePut(&$ch, $params)
  {
    $requestBody = $this->buildRequestBody($params);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $requestBody);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');

    return $this->curlExec($ch);
  }

  /**
   * execute with request-metod: DELETE
   *
   * @param curlHandler $ch
   * @param array $params
   * @return mixed 
   */
  protected function executeDelete(&$ch, $params)
  {
    $requestBody = $this->buildRequestBody($params);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $requestBody);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');

    return $this->curlExec($ch);
  }

}