<?php

/**
 * mo_viaebay__feedback_handler
 * 
 * handle responses from API-calls
 * 
 * @version   $Id: mo_viaebay__feedback_handler.php 161 2012-12-17 13:20:04Z $
 */

/**
 * handle responses from API-calls
 * 
 * @package   mo_viaebay 
 */
class mo_viaebay__feedback_handler
{

    /**
     * construct
     * 
     * @param oxConfig $oxConfig
     * @param oxSession $oxSession
     * @param array $viaebayConfig
     * @param mo_viaebay__logger $logger 
     */
    public function __construct($oxConfig, $oxSession, $viaebayConfig, mo_viaebay__logger $logger)
    {
        $this->oxConfig = $oxConfig;
        $this->oxSession = $oxSession;
        $this->logger = $logger;
        $this->viaebayConfig = $viaebayConfig;
    }

    /**
     * throw standard exception
     *
     * @param string $message
     * @throws @fn:oxNew 
     */
    protected function throwStandardException($message = '')
    {
        if (!$message) {
            $this->logger->logExecution();
        } else {
            $this->logger->error($message);
        }

        $e = oxNew('oxException', $message);
        throw $e;
    }

    /**
     * check if reponse contains required http codes
     *
     * @param array $response
     * @param mixed $codes
     * @return boolean 
     */
    protected function requireHttpCodes($response, $codes)
    {
        $codes = is_array($codes) ? $codes : array($codes);

        foreach ($codes as $entry) {
            if ($this->hasHttpCode($response, $entry)) {
                return true;
            }
        }

        $callers = debug_backtrace();
        $this->throwStandardException('Error in http-code check at ' . $callers[1]['function']);
    }

    /**
     * check authentication
     *
     * @param array $response
     * @throws @fn:oxNew 
     */
    protected function checkAuth($response)
    {
        if ($this->hasHttpCode($response, '401')) {
            $e = oxNew('oxConnectionException');
            throw $e;
        }
    }

    /**
     * check if reponse contains requirded http code
     *
     * @param array $response
     * @param string $code
     * @return boolean 
     */
    protected function hasHttpCode($response, $code)
    {
        $index = isset($response['Status Code']) ? 'Status Code' : 0;

        if (empty($response[$index])) {
            return false;
        }
        return strpos($response[$index], $code);
    }

    /**
     * handle get session cookie
     *
     * @param array $result
     * @return mixed boolean / exception 
     */
    public function handleGetSessionCookie($result)
    {
        $header = $this->decodeHeader($result['header']);
        if (empty($header['Set-Cookie'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }

        $this->oxSession->setVariable('mo_viaebay__session_cookie', $header['Set-Cookie']);
        return true;
    }

    /**
     * handle add category
     *
     * @param array $result
     * @return mixed 
     */
    public function handleAddCategory($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);

        $result = $this->decodeBody($result['body']);

        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }

        return $result['Id'];
    }

    /**
     * handle update category
     *
     * @param array $result
     * @return boolean 
     */
    public function handleUpdateCategory($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle get categories
     *
     * @param array $result
     * @return array 
     */
    public function handleGetCategories($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('200'));

        $result = $this->decodeBody($result['body']);

        return $result;
    }

    /**
     * handle get category relations
     *
     * @param array $result
     * @return array 
     */
    public function handleGetCategoryRelations($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);

        $result = $this->decodeBody($result['body']);
        return $result;
    }

    /**
     * handle set category relation
     *
     * @param array $result
     * @return boolean 
     */
    public function handleSetCategoryRelation($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);

        $this->requireHttpCodes($result, array('204', '500')); // 500 = link already exists !?

        return true;
    }

    /**
     * handle get category relation
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteCategoryRelation($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);

        $this->requireHttpCodes($result, array('204', '404')); //404 on delete allowed

        return true;
    }

    /**
     * handle delete category
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteCategory($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);

        $this->requireHttpCodes($result, array('204', '404'));

        return true;
    }

    /**
     * handle get articles
     *
     * @param array $result
     * @return array 
     */
    public function handleGetArticles($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);

        $this->requireHttpCodes($header, array('200'));

        $result = $this->decodeBody($result['body']);
        return $result;
    }

    /**
     * handle get article ids
     *
     * @param array $result
     * @return array 
     */
    public function handleGetArticleIds($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('200'));

        $result = $this->decodeBody($result['body']);
        return $result;
    }

    /**
     * handle add article
     *
     * @param array $result
     * @return array 
     */
    public function handleAddArticle($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $result = $this->decodeBody($result['body']);
        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }
        return $result['Id'];
    }

    /**
     * handle delete article
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteArticle($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle update article
     *
     * @param array $result
     * @return boolean 
     */
    public function handleUpdateArticle($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle delete article image
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteArticleImage($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle ass article image
     *
     * @param array $result
     * @return array 
     */
    public function handleAddArticleImage($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $result = $this->decodeBody($result['body']);
        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }
        return $result['Id'];
    }

    /**
     * handle add article category relation
     *
     * @param array $result
     * @return boolean 
     */
    public function handleAddArticleCategoryRelation($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle delete article category relation
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteArticleCategoryRelation($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle delete article attribute
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteArticleAttribute($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle update article-attribute
     *
     * @param array $result
     * @return boolean 
     */
    public function handleUpdateArticleAttribute($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle add article attribute
     *
     * @param array $result
     * @return mixed exception 
     */
    public function handleAddArticleAttribute($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $result = $this->decodeBody($result['body']);
        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }
        return $result['Id'];
    }

    /**
     * handle add extra Information
     *
     * @param array $result
     * @return mixed exception 
     */
    public function handleAddAdditionalInformation($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $result = $this->decodeBody($result['body']);
        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }
        return $result['Id'];
    }

    /**
     * handle add extra Information
     *
     * @param array $result
     * @return mixed exception 
     */
    public function handleUpdateAdditionalInformation($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $result = $this->decodeBody($result['body']);
        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }
        return $result['Id'];
    }

    /**
     * handle delete extra Information
     * @param type $result
     * @return boolean
     */
    public function handleDeleteAdditionalInformation($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle get orders
     *
     * @param array $result
     * @return array 
     */
    public function handleGetOrders($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('200'));

        $result = $this->decodeBody($result['body']);
        return $result;
    }

    /**
     * handle update order
     *
     * @param array $result
     * @return boolean 
     */
    public function handleUpdateOrder($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle report carrierUsed
     *
     * @param array $result
     * @return boolean 
     */
    public function handleReportCarrierUsed($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }
    
    /**
     * handle report payment
     *
     * @param array $result
     * @return boolean 
     */
    public function handleReportPayment($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle report shipment
     *
     * @param array $result
     * @return boolean 
     */
    public function handleReportShipment($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle report tracking code
     *
     * @param array $result
     * @return boolean 
     */
    public function handleReportTrackingcode($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204', '404'));
        return true;
    }

    /**
     * handle add article variant
     *
     * @param array $result
     * @return mixed 
     */
    public function handleAddArticleVariant($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);

        $result = $this->decodeBody($result['body']);

        if (empty($result['Id'])) {
            $this->throwStandardException('invalid ' . __FUNCTION__ . ' call');
        }

        return $result['Id'];
    }

    /**
     * handle update article variant
     *
     * @param array $result
     * @return boolean 
     */
    public function handleUpdateArticleVariant($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('204'));
        return true;
    }

    /**
     * handle delete article variant
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteArticleVariant($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle add article variant specifics
     *
     * @param array $result
     * @return boolean 
     */
    public function handleAddArticleVariantSpecific($result)
    {
        $header = $this->decodeHeader($result['header']);
        $this->checkAuth($header);
        $this->requireHttpCodes($header, array('201'));
        return true;
    }

    /**
     * handle delete article variant specific
     *
     * @param array $result
     * @return boolean 
     */
    public function handleDeleteArticleVariantSpecific($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('204', '404'));
        return true;
    }

    /**
     * handle set category relation
     *
     * @param array $result
     * @return boolean 
     */
    public function handleSetProductProfile($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('200'));
        return true;
    }

    /**
     * handle set category relation
     *
     * @param array $result
     * @return boolean 
     */
    public function handleUpdateInventory($result)
    {
        $result = $this->decodeHeader($result['header']);
        $this->checkAuth($result);
        $this->requireHttpCodes($result, array('200', '500', '404')); //500 also on price = 0
        return true;
    }

    /**
     * decode header
     *
     * @param array $header
     * @return array 
     */
    protected function decodeHeader($header)
    {
        $result = array();

        if (empty($header)) {
            return array();
        }

        foreach (explode("\n", $header) as $line) {
            //new key
            if (preg_match('#^([^:]+):(.*)$#', $line, $matches)) {
                $result[trim($matches[1])] = trim($matches[2]);
            }
            //append value to current array-element
            else {
                if ($key = key($result)) {
                    $result[$key] .= trim($line);
                } else {
                    $result[] = trim($line);
                }
            }
        }
        return $result;
    }

    /**
     * decode body
     *
     * @param array $body
     * @return array 
     */
    protected function decodeBody($body)
    {
        $result = json_decode($body, true);
        $result = $this->decodeParams($result);
        return isset($result['d']) ? $result['d'] : $result;
    }

    /**
     * check if utf8 encoding is chosen
     * 
     * @param array $params
     * @return array 
     */
    protected function decodeParams($params)
    {
        //reconvert feedback from UTF-8 to ISO
        if ($this->oxConfig->getConfigParam('mo_viaebay__encode_utf8')) {
            $params = mo_viaebay__main::getInstance()->getHelper()->convertCharset('UTF-8', 'ISO-8859-1', $params);
        }

        return $params;
    }

}
